//	A1.7.- i2c
//	Autores: Jess Mara Corres, Carlos Ruiz Zamarreo y Cndido Bariin
//	Pamplona, Agosto 2016

#include <htc.h>
#include "i2c.h"

// Configura el modulo I2C

void	i2c_init(){
	TRISC3 = 1;					// SLC
	TRISC4 = 1;					// SDA
	SSPADD = 0x0A;				// Reloj a 100KHz
	SSPSTAT = 0x80;			// Habilita el control de velocidad del mdulo
	SSPCON2 = 0x00;  			// Baja los flags
	SSPCON = 0x28;				// Modo maestro, mdulo I2C encendido
}

// Enva la secuencia de Start

void	i2c_start(){
	i2c_idle();					// Comprueba que est libre
	SSPCON2bits.SEN=1;			// Enva la condicin de inicio
	while(SSPCON2bits.SEN==1);		// Espera a que se enve
	while(SSPIF==0);			// Comprueba que se ha enviado
	SSPIF=0;					// Baja el flag
}

// Enva la secuencia de (re)start

void	i2c_restart(){
	SSPCON2bits.RSEN=1;			// Enva la secuencia restart
	while(SSPCON2bits.RSEN);		// Espera a que se enve
	while(SSPIF==0);			// Comprueba que se ha enviado
	SSPIF=0;					// Baja el flag
}
// 	Enva la secuencia de Stop

void	i2c_stop(){
	SSPCON2bits.PEN=1;			// Enva la secuencia de Stop
	while(SSPCON2bits.PEN);		// Espera a que se enve
	while(SSPIF==0);			// Comprueba que se ha enviado
	SSPIF=0;					// Baja el flag
}

// Comprueba que el modulo I2C est libre 

void	i2c_idle(){
while( ( SSPCON2 & 0x1F ) | SSPSTATbits.R_nW ); // Comprueba que el mdulo est libre
}

// Lee un byte	

unsigned char	i2c_read(){
	i2c_idle();
	SSPCON2bits.RCEN=1;			// Activa el modo recepcin
	while(SSPCON2bits.RCEN==1);		// Espera a recibir el byte
	while(SSPSTATbits.BF==0);		// Espera a que el buffer est lleno
	return SSPBUF;				// Devuelve el dato
}

// Enva un byte al esclavo y devuelve 0 en caso de error

unsigned char	i2c_write(unsigned char byte){
	i2c_idle();
	SSPBUF=byte;
	while(SSPSTAT&0x05);			// Comprueba si hay transmisin en curso
	if(SSPCON2bits.ACKSTAT==1){ 		// No ha recibido confirmacin
		SSPCON2bits.PEN=1;		// Enva secuencia de Stop
		return  0 ;				// devuelve 0
	}
	else{					// Ha recibido confirmacin
		while(SSPIF==0);			// Comprueba que ha transmitido
		SSPIF=0;					// Baja el flag
		return  1;					// Devuelve 1
	}
}

// Enva la secuencia Nack

void	i2c_sendnack(){
	SSPCON2bits.ACKDT=1;
	SSPCON2bits.ACKEN=1;
	while(SSPCON2bits.ACKEN==1);
}