//	A1.8.- lcd
//	Autores: Jess Mara Corres, Carlos Ruiz Zamarreo y Cndido Bariin
//	Pamplona, Agosto 2016

/*
 *	Rutinas para trabajar con la pantalla LCD Hitachi HD44780.
 *	Utiliza comunicaciones en modo 4 bit de la siguiente forma:
 *	
 *	PORTD bits 0-3 conectados a los bits de datos 4-7 de la LCD (high nibble)
 *	PORTD bit 4 conectado a la entrada LCD RS (seleccin de registro)
 *	PORTD bit 5 conectado a la entrada LCD RW (lectura/escritura)
 *	PORTD bit 6 conectado a la entrada LCD EN (reloj)
 *	PORTD bit 7 conectado a la entrada LCD ON bit (LCD power)
 */

#ifndef _XTAL_FREQ
 // Se asume una frecuencia de trabajo de 4MHz a menos que se haya especificado con anterioridad
 #define _XTAL_FREQ 4000000
#endif

#include	<htc.h>
#include	"lcd.h"

#define	LCD_RS RD4
#define	LCD_RW RD5
#define LCD_EN RD6

#define LCD_DATA	PORTD

#define	LCD_STROBE()	((LCD_EN = 1),(LCD_EN=0))

/* enva un byte a la LCD en modo 4 bits */

void lcd_write(unsigned char c){
	__delay_us(40);
	LCD_DATA = (LCD_DATA & 0xF0) | ( ( c >> 4 ) & 0x0F );
	LCD_STROBE();
	LCD_DATA = (LCD_DATA & 0xF0) | ( c & 0x0F );
	LCD_STROBE();
}

/* Limpia la pantalla LCD */
void lcd_clear(void){
	LCD_RS = 0;
	lcd_write(0x1);
	__delay_ms(2);
}

/* enva una cadena de caracteres a la LCD */
void lcd_puts(const char * s){
	LCD_RS = 1;	// write characters
	while(*s)
		lcd_write(*s++);
}

/* escribe un character en la LCD */
void lcd_putch(char c){
	LCD_RS = 1;	// write characters
	lcd_write( c );
}

/* Desplaza el cursor a la posicin indicada */
void lcd_goto(unsigned char pos){
	LCD_RS = 0;
	lcd_write(0x80+pos);
}
	
/* Inicializa la pantalla LCD en modo 4 bits */
void lcd_init(){
	char init_value;

	init_value = 0x3;
	TRISD=0;
	RD7=1; 							//enciende la pantalla LCD
	LCD_RS = 0;
	LCD_EN = 0;
	LCD_RW = 0;
	__delay_ms(15);					// espera 15ms
	LCD_DATA	 = (LCD_DATA & 0xF0) | init_value;	// enva el commando de inicio
	LCD_STROBE();
	__delay_ms(5);
	LCD_STROBE();
	__delay_us(200);
	LCD_STROBE();
	__delay_us(200);
	LCD_DATA = (LCD_DATA & 0xF0) | 2;			// Modo 4 bits
	LCD_STROBE();

	lcd_write(0x28); 					// Longitud del caracter
	lcd_write(0xF); 					// Display On, Cursor On, Cursor Blink
	lcd_clear();							// Limpia la pantalla
	lcd_write(0x6); 					// Modo de entrada On
}